import os

from bluer_options.help.functions import get_help
from bluer_objects import file, README

from bluer_journal import NAME, VERSION, ICON, REPO_NAME
from bluer_journal.help.functions import help_functions


def build():
    return all(
        README.build(
            path=os.path.join(file.path(__file__), readme["path"]),
            ICON=ICON,
            NAME=NAME,
            VERSION=VERSION,
            REPO_NAME=REPO_NAME,
            help_function=lambda tokens: get_help(
                tokens,
                help_functions,
                mono=True,
            ),
        )
        for readme in [
            {"path": ".."},
            {"path": "./docs"},
            # aliases
            {"path": "./docs/aliases"},
            {"path": "./docs/aliases/journal.md"},
        ]
    )



from typing import List

from bluer_options import env
from bluer_options.terminal import show_usage, xtra

from bluer_journal import ALIAS
from bluer_journal.help.git import help_functions as help_git
from bluer_journal.help.git import pull_options, push_options

args = [
    "[--checklist 0]",
    "[--relations 0]",
    "[--verbose 1]",
]


def help_sync(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra(
        "dryrun,{}".format(
            "offline" if env.BLUER_AI_WEB_STATUS == "online" else "~offline"
        ),
        mono=mono,
    )

    return show_usage(
        [
            "@journal",
            "sync",
            f"[{options}]",
            "[{}]".format(pull_options(mono=mono)),
            "[{}]".format(push_options(mono=mono, cascade=True)),
        ]
        + args,
        "sync journal.",
        mono=mono,
    )



import os
from typing import List, Tuple

from bluer_options import string
from bluer_objects import file
from bluer_objects.env import abcli_path_git

from bluer_journal import env
from bluer_journal.utils.latest import latest
from bluer_journal.logger import logger


def next(
    create: bool = True,
    verbose: bool = False,
) -> Tuple[bool, str]:
    suffix: int = 0
    while True:
        next_page: str = "dev-{}{}".format(
            string.pretty_date(
                as_filename=True,
                include_time=False,
                include_seconds=False,
            ),
            f"-{suffix:03d}" if suffix else "",
        )

        next_filename = os.path.join(
            abcli_path_git,
            f"{env.BLUER_JOURNAL_REPO}.wiki",
            f"{next_page}.md",
        )

        if not file.exists(next_filename):
            break

        suffix += 1

    if not create:
        return True, next_page

    # --

    success, latest_page = latest()
    if not success:
        return False, next_page

    logger.info(f"processing latest page: {latest_page}")
    latest_filename = os.path.join(
        abcli_path_git,
        f"{env.BLUER_JOURNAL_REPO}.wiki",
        f"{latest_page}.md",
    )

    success, page_content = file.load_text(
        latest_filename,
        log=verbose,
    )
    if not success:
        return False, next_page

    latest_page_content_updated: List[str] = []
    next_page_content: List[str] = [
        "---",
        "",
    ]
    fire_started: bool = False
    for line in page_content:
        if any(marker in line for marker in "🔥🎰"):
            fire_started = True

        if fire_started:
            next_page_content.append(line)
        else:
            latest_page_content_updated.append(line)

    latest_page_content_updated.append(f"--> [[{next_page}]]")

    if not file.save_text(
        latest_filename,
        latest_page_content_updated,
        log=verbose,
    ):
        return False, next_page

    # ---

    logger.info(f"processing next page: {next_page}")
    if not file.save_text(
        next_filename,
        next_page_content,
        log=verbose,
    ):
        return False, next_page

    # ---

    return True, next_page



from bluer_ai.help.generic import help_functions as generic_help_functions

from bluer_journal import ALIAS
from bluer_journal.help.git import help_functions as help_git
from bluer_journal.help.next import help_next
from bluer_journal.help.open import help_open
from bluer_journal.help.sync import help_sync


help_functions = generic_help_functions(plugin_name=ALIAS)


help_functions.update(
    {
        "git": help_git,
        "open": help_open,
        "next": help_next,
        "sync": help_sync,
    }
)



from typing import List

from bluer_options.terminal import show_usage, xtra

from bluer_journal.help.sync import args as sync_args
from bluer_journal.help.git import push_options


def help_next(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("~open", mono=mono)

    args = sorted(
        [
            "[--sync 0]]",
        ]
        + sync_args
    )

    return show_usage(
        [
            "@journal",
            "next",
            "[<title>]",
            f"[{options}]",
            "[{}]".format(
                push_options(
                    cascade=True,
                    mono=mono,
                )
            ),
        ]
        + args,
        "create the next page.",
        mono=mono,
    )
